package cd.casic.module.machine.component;

import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.module.machine.dal.dataobject.MachineInfoDO;
import cd.casic.module.machine.enums.AuthenticationType;
import cd.casic.module.machine.enums.ConnectionStatus;
import cd.casic.module.machine.enums.SSHChanelType;
import cd.casic.module.machine.service.SecretKeyService;
import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.Properties;

import static cd.casic.framework.commons.exception.util.ServiceExceptionUtil.exception;
import static cd.casic.module.machine.contants.MachineErrorCodeConstants.SECRET_KEY_NULL;
import static cd.casic.module.machine.contants.MachineErrorCodeConstants.*;

@Slf4j
@Data
@Component
public class WebSocketConnection {
    private SecretKeyService secretKeyService;

    private MachineInfoDO machineInfo;
    private ConnectionStatus connectionStatus = ConnectionStatus.DISCONNECTED;
    private Session sshSession;

    public WebSocketConnection(SecretKeyService secretKeyService) {
        this.secretKeyService = secretKeyService;
    }

    private static final int CONNECTION_TIMEOUT = 5000; // 连接超时时间(毫秒)

    public void initConnection(MachineInfoDO machineInfo) {
        try {
            this.machineInfo = machineInfo;
            this.sshSession = doConnect(machineInfo);
            log.info("已成功建立 SSH 连接至 {} ", machineInfo.getHostIp());
            this.connectionStatus = ConnectionStatus.CONNECTING;
        } catch (ServiceException e) {
            log.warn("SSH 连接失败: {}", e.getMessage());
            throw e;
        }
    }

    public void disconnect() {
        if (sshSession != null && sshSession.isConnected()) {
            try {
                sshSession.disconnect();
                log.info("SSH连接关闭: {}", machineInfo.getHostIp());
            } catch (Exception e) {
                log.error("关闭SSH连接失败: {}", e.getMessage());
                throw exception(CLOSE_CLOSE_SESSION_ERROR);
            }
        }
        connectionStatus = ConnectionStatus.DISCONNECTED;
    }

    /**
     * 执行远程命令，支持超时和中断处理
     */
    public void executeCommand(WebSocketSession webSocketSession, String command) {
        // 1. 检查连接状态
        if (sshSession == null || !sshSession.isConnected()) {
            sendErrorMessage(webSocketSession, "SSH连接未建立或已断开");
            return;
        }
        try {
            // 2. 创建SSH命令执行通道
            Channel channel;
            try {
                channel = sshSession.openChannel(SSHChanelType.EXEC.getMessage());
            } catch (JSchException e) {
                throw exception(CREATE_CHANEL_ERROR);
            }
            ((ChannelExec) channel).setCommand(command);
            // 3. 设置输入/输出流
            channel.setInputStream(null);
            ((ChannelExec) channel).setErrStream(System.err);
            // 4. 获取命令输出流
            InputStream inputStream = channel.getInputStream();
            InputStreamReader inputStreamReader = new InputStreamReader(inputStream, StandardCharsets.UTF_8);
            BufferedReader reader = new BufferedReader(inputStreamReader);
            // 5. 连接并执行命令
            channel.connect();
            // 6. 读取命令输出并实时发送给客户端
            String line;
            while ((line = reader.readLine()) != null) {
                // 实时发送输出到客户端
                webSocketSession.sendMessage(new TextMessage(line));
            }
            // 7. 等待命令执行完成
            int exitStatus = channel.getExitStatus();
            // 8. 发送命令执行完毕的消息
            webSocketSession.sendMessage(new TextMessage(
                "[系统] 命令执行完毕，退出状态: " + exitStatus
            ));
            // 9. 关闭通道
            channel.disconnect();
        } catch (JSchException | IOException e) {
            throw exception(EXECUTE_COMMAND_FAIL);
        }
    }

    // 发送错误消息的辅助方法
    public void sendErrorMessage(WebSocketSession webSocketSession, String message) {
        try {
            if (webSocketSession.isOpen()) {
                webSocketSession.sendMessage(new TextMessage("[错误] " + message));
            }
        } catch (IOException e) {
            log.error("发送错误消息失败", e);
            throw exception(WEBSOCKET_SEND_MESSAGE_ERROR);
        }
    }

    /**
     * 实际执行连接逻辑
     */
    private Session doConnect(MachineInfoDO machineInfo) {
        JSch jsch = new JSch();
        // 配置认证方式
        configureAuthentication(jsch, machineInfo);
        Session session;
        // 创建SSH会话
        try {
            session = jsch.getSession(
                machineInfo.getUsername(),
                machineInfo.getHostIp(),
                machineInfo.getSshPort() != null ? machineInfo.getSshPort() : 22
            );
        } catch (JSchException e) {
            throw exception(CREATE_SESSION_ERROR);
        }
        // 配置连接参数
        configureSession(session, machineInfo);
        // 建立连接
        try {
            session.connect(CONNECTION_TIMEOUT);
        } catch (JSchException e) {
            throw exception(SESSION_CONNECT_ERROR);
        }
        return session;
    }

    /**
     * 配置认证方式（密码或密钥）
     */
    private void configureAuthentication(JSch jsch, MachineInfoDO machineInfo) {
        if (machineInfo.getAuthenticationType() == AuthenticationType.SECRET_KEY.getCode()) {
            // 密钥认证
            if (machineInfo.getSecretKeyId() == null) {
                throw exception(SECRET_KEY_NULL);
            }
            String pubKeyContent = secretKeyService.getKeyContent(machineInfo.getSecretKeyId());
            // 验证秘钥格式
            if (!pubKeyContent.startsWith("-----BEGIN")) {
                log.error("无效的密钥格式{}", pubKeyContent);
                throw exception(INVALID_kEY_FORMAT);
            }
            try {
                // 尝试加载秘钥私钥
                jsch.addIdentity(
                    machineInfo.getName(),
                    pubKeyContent.getBytes(StandardCharsets.UTF_8),
                    null,
                    null
                );
                log.info("密钥加载成功 {}", machineInfo.getHostIp());
            } catch (JSchException e) {
                log.error("密钥加载失败: {}", e.getMessage());
                throw exception(READ_SECRET_CONTENT_ERROR);
            }
        } else if (machineInfo.getAuthenticationType() == AuthenticationType.PASSWORD.getCode()) {
            // 密码认证
            if (!StringUtils.hasText(machineInfo.getPassword())) {
                throw exception(PASSWORD_NOT_EXISTS);
            }
        } else {
            log.error("不支持该验证类型:{}", machineInfo.getAuthenticationType());
            throw exception(NOT_SUPPORT_AUTHENTICATION_TYPE);
        }
    }

    /**
     * 配置SSH会话参数（安全增强）
     */
    private void configureSession(Session session, MachineInfoDO machineInfo) {
        Properties config = new Properties();
        // 根据认证类型配置不同的认证策略
        if (machineInfo.getAuthenticationType() == 1) { // 密码认证
            // 设置密码
            session.setPassword(machineInfo.getPassword());
            config.put("StrictHostKeyChecking", "no");
            // 仅使用密码认证（禁用其他认证方式）
            config.put("PreferredAuthentications", "password");
            // 禁用公钥相关配置（避免干扰）
            config.put("PubkeyAuthentication", "no");
        } else { // 密钥认证
            try {
                String preKeyPath = secretKeyService.getSecretKey(machineInfo.getSecretKeyId()).getPath();
                JSch jsch = new JSch();
                jsch.addIdentity(preKeyPath); // 添加私钥
                // 保持默认认证顺序（公钥优先）
                config.put("PreferredAuthentications", "publicKey,password,keyboard-interactive");
            } catch (JSchException e) {
                log.error("SSH密钥配置失败", e);
                throw exception(SSH_KEY_CONFIGURATION_FAIL);
            }
        }
        config.put("ServerAliveInterval", "30"); // 每30秒发送一次心跳
        config.put("ServerAliveCountMax", "3"); // 允许3次心跳失败
        session.setConfig(config);
    }

}
