package cd.casic.module.machine.service.impl;

import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.module.machine.component.FileTreeComponent;
import cd.casic.module.machine.component.WebSocketSessionManager;
import cd.casic.module.machine.controller.vo.SecretKeyVO;
import cd.casic.module.machine.enums.AuthenticationType;
import cd.casic.module.machine.enums.MachineInfoType;
import cd.casic.module.machine.dal.mysql.MachineInfoMapper;
import cd.casic.module.machine.controller.vo.MachineInfoVO;
import cd.casic.module.machine.dal.dataobject.MachineInfoDO;
import cd.casic.module.machine.enums.ConnectionStatus;
import cd.casic.module.machine.enums.MachineInfoStatus;
import cd.casic.module.machine.component.WebSocketConnection;
import cd.casic.module.machine.service.MachineInfoService;
import cd.casic.module.machine.service.SecretKeyService;
import com.google.common.annotations.VisibleForTesting;
import com.jcraft.jsch.Session;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import cd.casic.framework.commons.util.object.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

import static cd.casic.framework.commons.exception.util.ServiceExceptionUtil.exception;
import static cd.casic.module.machine.contants.MachineErrorCodeConstants.*;

/**
 * 机器信息服务实现类
 */
@Slf4j
@Service("machineInfoService")
public class MachineInfoServiceImpl implements MachineInfoService {

    @Resource
    private SecretKeyService secretKeyService;

    @Resource
    private MachineInfoMapper machineInfoMapper;

    @Resource
    private FileTreeComponent fileTreeComponent;

    @Override
    public Long createMachine(MachineInfoVO machineInfoVO) {
        validateMachineEnvAdd(machineInfoVO);
        validateMachineTagUnique(machineInfoVO.getTag());
        MachineInfoDO machineInfoDO = BeanUtils.toBean(machineInfoVO, MachineInfoDO.class);
        if (machineInfoVO.getAuthenticationType() == 2) {
            Long secretKeyId = machineInfoDO.getSecretKeyId();
            SecretKeyVO secretKey = secretKeyService.getSecretKey(secretKeyId);
            if (secretKey == null) {
                throw exception(SECRET_KEY_NOT_EXISTS);
            }
        }
        machineInfoMapper.insert(machineInfoDO);
        return machineInfoDO.getId();
    }

    @Override
    public void updateMachineInfo(MachineInfoVO machineInfoVO) {
        validateMachineEnvAdd(machineInfoVO);
        String newTag = machineInfoVO.getTag();
        MachineInfoDO machineInfoDO = validateMachineInfoExists(machineInfoVO.getId());
        String oldTag = machineInfoDO.getTag();
        if (!newTag.equals(oldTag)) {
            validateMachineTagUnique(newTag);
        }
        BeanUtils.copyProperties(machineInfoVO, machineInfoDO);
        machineInfoMapper.updateById(machineInfoDO);
    }

    @Override
    public Integer updateStatus(MachineInfoVO machineInfoVO) {
        machineInfoMapper.updateStatus(machineInfoVO.getId(), machineInfoVO.getStatus());
        return machineInfoVO.getStatus();
    }

    @Override
    public PageResult<MachineInfoDO> listMachineInfo(MachineInfoVO machineInfoVO) {
        return machineInfoMapper.selectPage(machineInfoVO);
    }

    @Override
    public void bindingSecretKey(List<Long> machineInfoIds, Long secretKeyId) {
        machineInfoMapper.bindingSecretKey(machineInfoIds, secretKeyId);
    }

    @Override
    @Transactional//其中一个在线，那么就回滚
    public void deleteMachineInfoList(String machineInfoIds) {
        List<Long> machineInfoIdList = Arrays.stream(machineInfoIds.split(","))
            .map(String::trim)
            .filter(s -> !s.isEmpty())
            .map(Long::parseLong)
            .toList();
        machineInfoIdList.forEach(this::deleteMachineInfo);
    }

    @Override
    public void deleteMachineInfo(Long machineInfoId) {
        MachineInfoDO machineInfoDO = validateMachineInfoExists(machineInfoId);
        validateMachineEnable(machineInfoDO);
        machineInfoMapper.deleteById(machineInfoId);
    }


    @Override
    public boolean testConnection(Long id) {
        //先查询机器是否存在，在判断机器可用性
        MachineInfoDO machineInfoDO = validateMachineInfoExists(id);
        validateMachineUnEnable(machineInfoDO);
        log.info("测试机器连接: {}", machineInfoDO.getHostIp());
        WebSocketConnection webSocketConnection = createWebSocketConnection(machineInfoDO);
        webSocketConnection.initConnection(machineInfoDO);
        return true;
    }

    @Override
    public Map<String, Object> connect(Long id) {
        //todo使用代理机器的情况还未完成
        WebSocketConnection webSocketConnection = new WebSocketConnection(this.secretKeyService);
        MachineInfoDO machineInfoDO = validateMachineInfoExists(id);
        //初始化连接
        webSocketConnection.initConnection(machineInfoDO);
        WebSocketSessionManager.addWebSocketConnectionByMachineId(id, webSocketConnection);
        return fileTreeComponent.getRemoteFileTree(webSocketConnection.getSshSession(), "/");
    }

    @Override
    public ConnectionStatus getConnectionStatus(Long id) {
        validateMachineInfoExists(id);
        WebSocketConnection webSocketConnection = WebSocketSessionManager.getWebSocketConnectionByMachineId(id);
        return webSocketConnection == null ? ConnectionStatus.DISCONNECTED : webSocketConnection.getConnectionStatus();
    }

    @Override
    public Map<Long, ConnectionStatus> getAllConnectionStatus() {
        return WebSocketSessionManager.getAllWebSocketConnections().entrySet().stream()
            .collect(Collectors.toMap(
                Map.Entry::getKey,
                entry -> entry.getValue().getConnectionStatus()
            ));
    }

//    @Override
//    public boolean uploadFile(String sessionId, String localFilePath, String remoteFilePath) {
//        log.info("上传文件: {} -> {}, 会话ID: {}", localFilePath, remoteFilePath, sessionId);
//        WebSocketConnection webSocketConnection = sessionConnectionMap.get(sessionId);
//        if (webSocketConnection == null||webSocketConnection.getConnectionStatus() != ConnectionStatus.CONNECTED) {
//            throw exception(SESSION_NOT_CONNECT);
//        }
//        try {
//            return webSocketConnection.uploadFile(localFilePath, remoteFilePath);
//        } catch (Exception e) {
//            log.error("文件上传失败: {}", e.getMessage(), e);
//            throw exception(FILE_UPLOAD_ERROR);
//        }
//    }

//    @Override
//    public boolean downloadFile(String sessionId, String remoteFilePath, String localFilePath) {
//        log.info("下载文件: {} -> {}, 会话ID: {}", remoteFilePath, localFilePath, sessionId);
//
//        WebSocketConnection webSocketConnection = sessionConnectionMap.get(sessionId);
//        if (webSocketConnection == null||webSocketConnection.getConnectionStatus() != ConnectionStatus.CONNECTED) {
//            throw new RuntimeException("会话不存在: " + sessionId);
//        }
//        try {
//            return webSocketConnection.downloadFile(remoteFilePath, localFilePath);
//        } catch (Exception e) {
//            log.error("文件下载失败: {}", e.getMessage(), e);
//            throw exception(FILE_DOWNLOAD_ERROR);
//        }
//    }

    @VisibleForTesting
    void validateMachineEnvAdd(MachineInfoVO machineInfoVO) {
        if (machineInfoVO.getHostIp().isEmpty()) {
            throw exception(MACHINE_INFO_HOST_IP_NULL);
        }
        if (machineInfoVO.getUsername().isEmpty()) {
            throw exception(MACHINE_INFO_USER_NAME_NULL);
        }
        if (machineInfoVO.getTag().isEmpty()) {
            throw exception(MACHINE_INFO_TAG_NULL);
        }

        if (machineInfoVO.getAuthenticationType() != null) {
            boolean flag = true;
            for (int type : AuthenticationType.ARRAYS) {
                if (type == machineInfoVO.getAuthenticationType()) {
                    flag = false;
                    break;
                }
            }
            if (flag) {
                throw exception(MACHINE_INFO_AUTHENTICATION_TYPE_NOT_EXISTS);
            }
        } else {
            throw exception(MACHINE_INFO_AUTHENTICATION_TYPE_NULL);
        }


        if (machineInfoVO.getMachineInfoType() != null) {
            boolean flag = true;
            for (int type : MachineInfoType.ARRAYS) {
                if (type == machineInfoVO.getMachineInfoType()) {
                    flag = false;
                    break;
                }
            }
            if (flag) {
                throw exception(MACHINE_INFO_TYPE_NOT_EXISTS);
            }
        } else {
            throw exception(MACHINE_INFO_TYPE_NULL);
        }

    }

    @VisibleForTesting
    void validateMachineTagUnique(String tag) {
        if (machineInfoMapper.existsByTag(tag)) {
            throw exception(MACHINE_INFO_TAG_EXISTS);
        }
    }

    @Override
    public MachineInfoDO validateMachineInfoExists(Long id) {
        if (id == null) {
            throw exception(MACHINE_INFO_NULL);
        }
        MachineInfoDO machineInfoDO = machineInfoMapper.selectById(id);
        if (machineInfoDO == null) {
            throw exception(MACHINE_INFO_NULL);
        }
        return machineInfoDO;
    }

    @Override
    public Map<String, Object> fileTreeNode(Long machineId, String path) {
        validateMachineInfoExists(machineId);
        Session sshSession = WebSocketSessionManager.getWebSocketConnectionByMachineId(machineId).getSshSession();
        return fileTreeComponent.getRemoteFileTree(sshSession, path);
    }

    @VisibleForTesting
    void validateMachineEnable(MachineInfoDO machineInfoDO) {
        if (machineInfoDO.getStatus() == MachineInfoStatus.ENABLE.getCode()) {
            throw exception(MACHINE_ENABLE);
        }
    }

    @VisibleForTesting
    void validateMachineUnEnable(MachineInfoDO machineInfoDO) {

        if (machineInfoDO.getStatus() == MachineInfoStatus.UN_ENABLE.getCode()) {
            throw exception(MACHINE_UN_ENABLE);
        }
    }

    @VisibleForTesting
    WebSocketConnection createWebSocketConnection(MachineInfoDO machineInfoDO) {
        if (WebSocketSessionManager.containsMachineId(machineInfoDO.getId())) {
            return WebSocketSessionManager.getWebSocketConnectionByMachineId((machineInfoDO.getId()));
        } else {
            return new WebSocketConnection(this.secretKeyService);
        }
    }
}
