package cd.casic.module.machine.service.impl;

import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.framework.commons.util.object.BeanUtils;
import cd.casic.module.machine.controller.vo.SecretKeyVO;
import cd.casic.module.machine.dal.dataobject.MachineInfoDO;
import cd.casic.module.machine.dal.dataobject.SecretKeyDO;
import cd.casic.module.machine.dal.mysql.MachineInfoMapper;
import cd.casic.module.machine.dal.mysql.SecretKeyMapper;
import cd.casic.module.machine.service.MachineInfoService;
import cn.hutool.core.io.resource.ResourceUtil;
import cn.hutool.core.util.IdUtil;
import cd.casic.module.machine.utils.AliYunOssClient;
import cd.casic.module.machine.service.SecretKeyService;
import com.google.common.annotations.VisibleForTesting;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StreamUtils;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.List;

import static cd.casic.framework.commons.exception.util.ServiceExceptionUtil.exception;
import static cd.casic.module.machine.contants.MachineErrorCodeConstants.*;

/**
 * 密钥服务实现类
 */
@Slf4j
@Service("secretKeyService")
public class SecretKeyServiceImpl implements SecretKeyService {

    @Resource
    private MachineInfoService machineInfoService;

    @Resource
    private AliYunOssClient aliYunOssClient;

    @Resource
    private SecretKeyMapper secretKeyMapper;

    @Resource
    private MachineInfoMapper machineInfoMapper;

    @Override
    public SecretKeyVO getSecretKey(Long id) {
        SecretKeyDO secretKeyDO = validateSecretKeyExists(id);
        return BeanUtils.toBean(secretKeyDO, SecretKeyVO.class);
    }

    @Override
    public void deleteSecretKey(Long id) {
        validateSecretKeyExists(id);
        secretKeyMapper.deleteById(id);
    }

    @Override
    public void unbindMachine(SecretKeyVO secretKeyVO) {
        machineInfoService.bindingSecretKey(secretKeyVO.getMachineInfoIds(), null);
    }

    @Override
    public ResponseEntity<InputStreamResource> downloadSecretFile(Long id) {
        // 获取私钥内容
        String keyContent = getKeyContent(id);
        if (keyContent == null || keyContent.isEmpty()) {
            throw exception(SECRET_KEY_NULL);
        }
        // 将字符串转换为输入流
        ByteArrayInputStream inputStream = new ByteArrayInputStream(keyContent.getBytes(StandardCharsets.UTF_8));
        // 设置响应头
        HttpHeaders headers = new HttpHeaders();
        headers.add(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=private_key.pem");
        headers.add(HttpHeaders.CONTENT_TYPE, "application/octet-stream");
        headers.add(HttpHeaders.CONTENT_LENGTH, String.valueOf(keyContent.getBytes(StandardCharsets.UTF_8).length));
        // 返回带有文件流的响应实体
        return ResponseEntity.ok()
            .headers(headers)
            .body(new InputStreamResource(inputStream));
    }

    @Override
    public List<MachineInfoDO> getBindMachine(Long secretKeyId) {
        return machineInfoMapper.selectBindMachineBySecretKey(secretKeyId);
    }

    @Override
    public Long createSecretKey(SecretKeyVO secretKeyVO) {
        validateSecretKeyAdd(secretKeyVO);
        String localPath = secretKeyVO.getPath();
        //将反斜杠替换为双反斜杠（Windows路径转义）
        localPath = localPath.replace("\\", "\\\\");
        String ossPath = upLoadSecretKey(localPath);
        //检查得到的oss路径是否为空
        validateSecretKeyPath(ossPath);
        secretKeyVO.setPath(ossPath);
        SecretKeyDO secretKeyDO = BeanUtils.toBean(secretKeyVO, SecretKeyDO.class);
        secretKeyMapper.insert(secretKeyDO);
        return secretKeyDO.getId();
    }

    @Override
    public void updateSecretKey(SecretKeyVO secretKeyVO) {
        SecretKeyDO secretKeyDO = validateSecretKeyExists(secretKeyVO.getId());
        //路径改变==改变密钥
        if (!secretKeyDO.getPath().equals(secretKeyVO.getPath())) {
            String ossPath = upLoadSecretKey(secretKeyVO.getPath());
            BeanUtils.copyProperties(secretKeyVO, secretKeyDO);
            secretKeyDO.setPath(ossPath);
        } else {
            BeanUtils.copyProperties(secretKeyVO, secretKeyDO);
        }
        secretKeyMapper.updateById(secretKeyDO);
    }

    @Override
    public void bindingMachine(SecretKeyVO secretKeyVO) {
        validateSecretKeyExists(secretKeyVO.getId());
        machineInfoService.bindingSecretKey(secretKeyVO.getMachineInfoIds(), secretKeyVO.getId());
    }

    @Override
    @Transactional
    public void deleteSecretKeyList(List<Long> ids) {
        ids.forEach(
            secretKeyId -> {
                SecretKeyDO secretKeyDO = validateSecretKeyExists(secretKeyId);
                if (secretKeyDO.getPath() != null && !secretKeyDO.getPath().isEmpty()) {
                    try {
                        //文件名
                        //删除子目录文件，需要在前面加上根目录文件路径
                        String fileName = secretKeyDO.getPath().substring(secretKeyDO.getPath().lastIndexOf("/") + 1);
                        aliYunOssClient.delete(fileName);
                    } catch (Exception e) {
                        throw exception(DELETE_FILE_FAIL);
                    }
                }
            }
        );

        //绑定的机器全部设置为空
        machineInfoService.bindingSecretKey(ids, null);
        secretKeyMapper.deleteBatchIds(ids);
    }

    @Override
    public PageResult<SecretKeyDO> getSecretKeypage(SecretKeyVO secretKeyVO) {
        return secretKeyMapper.selectPage(secretKeyVO);
    }

    public String upLoadSecretKey(String localPath) {
        //使用S3FileClient上传文件
        aliYunOssClient.init();
        //传输到指定文件，需要在path前面加上文件路径
        String path = IdUtil.fastSimpleUUID() + ".txt";
        //上传文件是从本地上传，这里传的是本地文件地址
        byte[] content = ResourceUtil.readBytes(localPath);
        String ossPath;
        try {
            ossPath = aliYunOssClient.upload(content, path, "txt");
        } catch (Exception e) {
            throw exception(UPLOADING_FILE_FAIL);
        }
        return ossPath;
    }

    @VisibleForTesting
    void validateSecretKeyAdd(SecretKeyVO secretKeyVO) {
        if (secretKeyVO == null) {
            throw exception(SECRET_KEY_NULL);
        }
        if (secretKeyVO.getPath().isEmpty()) {
            throw exception(SECRET_KEY_PATH_NULL);
        }
    }

    @VisibleForTesting
    void validateSecretKeyPath(String path) {
        if (path.isEmpty()) {
            throw exception(SECRET_KEY_PATH_NULL);
        }
    }


    @VisibleForTesting
    SecretKeyDO validateSecretKeyExists(Long id) {
        if (id == null) {
            throw exception(SECRET_KEY_NOT_EXISTS);
        }
        SecretKeyDO secretKeyDO = secretKeyMapper.selectById(id);
        if (secretKeyDO == null) {
            throw exception(SECRET_KEY_NOT_EXISTS);
        }
        return secretKeyDO;
    }

    @Override
    public String getKeyContent(Long secretKeyId) {
        if (secretKeyId == null) {
            return null;
        }
        SecretKeyVO secretKey = getSecretKey(secretKeyId);
        byte[] content;
        try {
            content = aliYunOssClient.getContent(secretKey.getPath().substring(secretKey.getPath().lastIndexOf("/") + 1));
        } catch (Exception e) {
            log.error("读取密钥文件失败", e);
            throw exception(READ_SECRET_CONTENT_ERROR);
        }
        //改为S3FileClient读取
        InputStream read = new ByteArrayInputStream(content);
        try {
            return StreamUtils.copyToString(read, StandardCharsets.UTF_8);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
